# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import segment
from neutron_lib.api.definitions import subnet


# The alias of the extension.
ALIAS = 'subnet-segmentid-writable'

# Whether or not this extension is simply signaling behavior to the user
# or it actively modifies the attribute map.
IS_SHIM_EXTENSION = False

# Whether the extension is marking the adoption of standardattr model for
# legacy resources, or introducing new standardattr attributes. False or
# None if the standardattr model is adopted since the introduction of
# resource extension.
# If this is True, the alias for the extension should be prefixed with
# 'standard-attr-'.
IS_STANDARD_ATTR_EXTENSION = False

# The name of the extension.
NAME = 'Subnet SegmentID (writable)'

# A prefix for API resources. An empty prefix means that the API is going
# to be exposed at the v2/ level as any other core resource.
API_PREFIX = ''

# The description of the extension.
DESCRIPTION = (
    "Provides a writable segment_id attribute for a subnet resource.")

# A timestamp of when the extension was introduced.
UPDATED_TIMESTAMP = "2018-03-12T00:00:00-00:00"


RESOURCE_ATTRIBUTE_MAP = {
    subnet.COLLECTION_NAME: {
        segment.SEGMENT_ID: {'allow_post': True, 'allow_put': True,
                             'default': None,
                             'validate': {'type:uuid_or_none': None},
                             'is_visible': True}
    }
}

# The subresource attribute map for the extension. It adds child resources
# to main extension's resource. The subresource map must have a parent and
# a parameters entry. If an extension does not need such a map, None can
# be specified (mandatory).
SUB_RESOURCE_ATTRIBUTE_MAP = {}

# The action map: it associates verbs with methods to be performed on
# the API resource.
ACTION_MAP = {}

# The action status.
ACTION_STATUS = {
}

# The list of required extensions.
REQUIRED_EXTENSIONS = [segment.ALIAS]

# The list of optional extensions.
OPTIONAL_EXTENSIONS = []



# Copyright 2013 VMware, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from webob import exc

from neutron_lib._i18n import _
from neutron_lib.api import validators
from neutron_lib.exceptions import allowedaddresspairs as exceptions


def _validate_allowed_address_pairs(address_pairs, valid_values=None):
    """Validates a list of allowed address pair dicts.

    Validation herein requires the caller to have registered the
    max_allowed_address_pair oslo config option in the global CONF prior
    to having this validator used.

    :param address_pairs: A list of address pair dicts to validate.
    :param valid_values: Not used.
    :returns: None
    :raises: AllowedAddressPairExhausted if the address pairs requested
    exceeds cfg.CONF.max_allowed_address_pair. AllowedAddressPairsMissingIP
    if any address pair dicts are missing and IP address.
    DuplicateAddressPairInRequest if duplicated IPs are in the list of
    address pair dicts. Otherwise a HTTPBadRequest is raised if any of
    the address pairs are invalid.
    """
    unique_check = {}
    if not isinstance(address_pairs, list):
        raise exc.HTTPBadRequest(
            _("Allowed address pairs must be a list."))
    if len(address_pairs) > cfg.CONF.max_allowed_address_pair:
        raise exceptions.AllowedAddressPairExhausted(
            quota=cfg.CONF.max_allowed_address_pair)

    for address_pair in address_pairs:
        msg = validators.validate_dict(address_pair)
        if msg:
            return msg
        # mac_address is optional, if not set we use the mac on the port
        if 'mac_address' in address_pair:
            msg = validators.validate_mac_address(address_pair['mac_address'])
            if msg:
                raise exc.HTTPBadRequest(msg)
        if 'ip_address' not in address_pair:
            raise exceptions.AllowedAddressPairsMissingIP()

        mac = address_pair.get('mac_address')
        ip_address = address_pair['ip_address']
        if (mac, ip_address) not in unique_check:
            unique_check[(mac, ip_address)] = None
        else:
            raise exceptions.DuplicateAddressPairInRequest(
                mac_address=mac, ip_address=ip_address)

        invalid_attrs = set(address_pair.keys()) - set(['mac_address',
                                                        'ip_address'])
        if invalid_attrs:
            msg = (_("Unrecognized attribute(s) '%s'") %
                   ', '.join(set(address_pair.keys()) -
                             set(['mac_address', 'ip_address'])))
            raise exc.HTTPBadRequest(msg)

        if '/' in ip_address:
            msg = validators.validate_subnet(ip_address)
        else:
            msg = validators.validate_ip_address(ip_address)
        if msg:
            raise exc.HTTPBadRequest(msg)



# Copyright (c) 2015 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import hashlib
from unittest import mock

from oslo_utils import excutils
from oslo_utils import uuidutils

from neutron_lib.api.definitions import portbindings_extended as pb_ext
from neutron_lib import constants
from neutron_lib import exceptions
from neutron_lib.plugins import utils
from neutron_lib.tests import _base as base


LONG_NAME1 = "A_REALLY_LONG_INTERFACE_NAME1"
LONG_NAME2 = "A_REALLY_LONG_INTERFACE_NAME2"
SHORT_NAME = "SHORT"
MOCKED_HASH = "mockedhash"


class MockSHA(object):
    def hexdigest(self):
        return MOCKED_HASH


class TestUtils(base.BaseTestCase):

    def test_is_valid_vlan_tag(self):
        for v in [constants.MIN_VLAN_TAG, constants.MIN_VLAN_TAG + 2,
                  constants.MAX_VLAN_TAG, constants.MAX_VLAN_TAG - 2]:
            self.assertTrue(utils.is_valid_vlan_tag(v))

    def test_is_valid_vlan_tag_invalid_data(self):
        for v in [constants.MIN_VLAN_TAG - 1, constants.MIN_VLAN_TAG - 2,
                  constants.MAX_VLAN_TAG + 1, constants.MAX_VLAN_TAG + 2]:
            self.assertFalse(utils.is_valid_vlan_tag(v))

    def test_verify_vlan_range(self):
        for v in [(constants.MIN_VLAN_TAG, constants.MIN_VLAN_TAG + 2),
                  (constants.MIN_VLAN_TAG + 2, constants.MAX_VLAN_TAG - 2)]:
            self.assertIsNone(utils.verify_vlan_range(v))

    def test_verify_vlan_range_invalid_range(self):
        for v in [(constants.MIN_VLAN_TAG, constants.MAX_VLAN_TAG + 2),
                  (constants.MIN_VLAN_TAG + 4, constants.MIN_VLAN_TAG + 1)]:
            self.assertRaises(exceptions.NetworkVlanRangeError,
                              utils.verify_vlan_range, v)

    def test_parse_network_vlan_range(self):
        self.assertEqual(
            ('n1', (1, 3)),
            utils.parse_network_vlan_range('n1:1:3'))
        self.assertEqual(
            ('n1', (1, 1111)),
            utils.parse_network_vlan_range('n1:1:1111'))

    def test_parse_network_vlan_range_invalid_range(self):
        self.assertRaises(exceptions.NetworkVlanRangeError,
                          utils.parse_network_vlan_range,
                          'n1:1,4')

    def test_parse_network_vlan_range_missing_network(self):
        self.assertRaises(exceptions.PhysicalNetworkNameError,
                          utils.parse_network_vlan_range,
                          ':1:4')

    def test_parse_network_vlan_range_invalid_min_type(self):
        self.assertRaises(exceptions.NetworkVlanRangeError,
                          utils.parse_network_vlan_range,
                          'n1:a:4')

    def test_parse_network_vlan_ranges(self):
        ranges = utils.parse_network_vlan_ranges(
            ['n1:1:3', 'n2:2:4', 'n3', 'n4', 'n4:10:12'])
        self.assertEqual(4, len(ranges.keys()))
        self.assertIn('n1', ranges.keys())
        self.assertIn('n2', ranges.keys())
        self.assertEqual(2, len(ranges['n1'][0]))
        self.assertEqual(1, ranges['n1'][0][0])
        self.assertEqual(3, ranges['n1'][0][1])
        self.assertEqual(2, len(ranges['n2'][0]))
        self.assertEqual(2, ranges['n2'][0][0])
        self.assertEqual(4, ranges['n2'][0][1])
        self.assertEqual([constants.VLAN_VALID_RANGE], ranges['n3'])
        self.assertEqual([constants.VLAN_VALID_RANGE], ranges['n4'])

    def test_is_valid_gre_id(self):
        for v in [constants.MIN_GRE_ID, constants.MIN_GRE_ID + 2,
                  constants.MAX_GRE_ID, constants.MAX_GRE_ID - 2]:
            self.assertTrue(utils.is_valid_gre_id(v))

    def test_is_valid_gre_id_invalid_id(self):
        for v in [constants.MIN_GRE_ID - 1, constants.MIN_GRE_ID - 2,
                  True, 'z', 99.999, []]:
            self.assertFalse(utils.is_valid_gre_id(v))

    def test_is_valid_vxlan_vni(self):
        for v in [constants.MIN_VXLAN_VNI, constants.MAX_VXLAN_VNI,
                  constants.MIN_VXLAN_VNI + 1, constants.MAX_VXLAN_VNI - 1]:
            self.assertTrue(utils.is_valid_vxlan_vni(v))

    def test_is_valid_vxlan_vni_invalid_values(self):
        for v in [constants.MIN_VXLAN_VNI - 1, constants.MAX_VXLAN_VNI + 1,
                  True, 'a', False, {}]:
            self.assertFalse(utils.is_valid_vxlan_vni(v))

    def test_is_valid_geneve_vni(self):
        for v in [constants.MIN_GENEVE_VNI, constants.MAX_GENEVE_VNI,
                  constants.MIN_GENEVE_VNI + 1, constants.MAX_GENEVE_VNI - 1]:
            self.assertTrue(utils.is_valid_geneve_vni(v))

    def test_is_valid_geneve_vni_invalid_values(self):
        for v in [constants.MIN_GENEVE_VNI - 1, constants.MAX_GENEVE_VNI + 1,
                  True, False, (), 'True']:
            self.assertFalse(utils.is_valid_geneve_vni(v))

    def test_verify_tunnel_range_known_tunnel_type(self):
        mock_fns = [mock.Mock(return_value=False) for _ in range(3)]
        mock_map = {
            constants.TYPE_GRE: mock_fns[0],
            constants.TYPE_VXLAN: mock_fns[1],
            constants.TYPE_GENEVE: mock_fns[2]
        }

        with mock.patch.dict(utils._TUNNEL_MAPPINGS, mock_map):
            for t in [constants.TYPE_GRE, constants.TYPE_VXLAN,
                      constants.TYPE_GENEVE]:
                self.assertRaises(
                    exceptions.NetworkTunnelRangeError,
                    utils.verify_tunnel_range, [0, 1], t)
            for f in mock_fns:
                f.assert_called_once_with(0)

    def test_verify_tunnel_range_invalid_range(self):
        for r in [[1, 0], [0, -1], [2, 1]]:
            self.assertRaises(
                exceptions.NetworkTunnelRangeError,
                utils.verify_tunnel_range, r, constants.TYPE_FLAT)

    def test_verify_tunnel_range(self):
        for r in [[0, 1], [-1, 0], [1, 2]]:
            self.assertIsNone(
                utils.verify_tunnel_range(r, constants.TYPE_FLAT))

    def test_delete_port_on_error(self):
        core_plugin = mock.Mock()
        with mock.patch.object(excutils, 'save_and_reraise_exception'):
            with mock.patch.object(utils, 'LOG'):
                with utils.delete_port_on_error(core_plugin, 'ctx', '1'):
                    raise Exception()

        core_plugin.delete_port.assert_called_once_with(
            'ctx', '1', l3_port_check=False)

    def test_update_port_on_error(self):
        core_plugin = mock.Mock()
        with mock.patch.object(excutils, 'save_and_reraise_exception'):
            with mock.patch.object(utils, 'LOG'):
                with utils.update_port_on_error(core_plugin, 'ctx', '1', '2'):
                    raise Exception()

        core_plugin.update_port.assert_called_once_with(
            'ctx', '1', {'port': '2'})

    @mock.patch.object(hashlib, 'sha1', return_value=MockSHA())
    def test_get_interface_name(self, mock_sha1):
        prefix = "pre-"
        prefix_long = "long_prefix"
        prefix_exceeds_max_dev_len = "much_too_long_prefix"
        hash_used = MOCKED_HASH[0:6]

        self.assertEqual("A_REALLY_" + hash_used,
                         utils.get_interface_name(LONG_NAME1))
        self.assertEqual("SHORT",
                         utils.get_interface_name(SHORT_NAME))
        self.assertEqual("pre-A_REA" + hash_used,
                         utils.get_interface_name(LONG_NAME1, prefix=prefix))
        self.assertEqual("pre-SHORT",
                         utils.get_interface_name(SHORT_NAME, prefix=prefix))
        # len(prefix) > max_device_len - len(hash_used)
        self.assertRaises(ValueError, utils.get_interface_name, SHORT_NAME,
                          prefix_long)
        # len(prefix) > max_device_len
        self.assertRaises(ValueError, utils.get_interface_name, SHORT_NAME,
                          prefix=prefix_exceeds_max_dev_len)

    def test_get_interface_uniqueness(self):
        prefix = "prefix-"
        if_prefix1 = utils.get_interface_name(LONG_NAME1, prefix=prefix)
        if_prefix2 = utils.get_interface_name(LONG_NAME2, prefix=prefix)
        self.assertNotEqual(if_prefix1, if_prefix2)

    @mock.patch.object(hashlib, 'sha1', return_value=MockSHA())
    def test_get_interface_max_len(self, mock_sha1):
        self.assertEqual(constants.DEVICE_NAME_MAX_LEN,
                         len(utils.get_interface_name(LONG_NAME1)))
        self.assertEqual(10, len(utils.get_interface_name(LONG_NAME1,
                                                          max_len=10)))
        self.assertEqual(12, len(utils.get_interface_name(LONG_NAME1,
                                                          prefix="pre-",
                                                          max_len=12)))

    def test_create_network(self):
        mock_plugin = mock.Mock()
        mock_plugin.create_network = lambda c, n: n
        mock_ctx = mock.Mock()
        mock_ctx.project_id = 'p1'
        net = utils.create_network(
            mock_plugin, mock_ctx, {'network': {'name': 'n1'}})
        self.assertDictEqual(
            {'project_id': 'p1', 'admin_state_up': True, 'shared': False,
             'tenant_id': 'p1', 'name': 'n1'},
            net['network'])

    def test_create_subnet(self):
        mock_plugin = mock.Mock()
        mock_plugin.create_subnet = lambda c, s: s
        mock_ctx = mock.Mock()
        mock_ctx.project_id = 'p1'
        net_id = uuidutils.generate_uuid()
        snet = utils.create_subnet(
            mock_plugin, mock_ctx,
            {'subnet': {'network_id': net_id, 'ip_version': 4}})
        self.assertEqual('p1', snet['subnet']['tenant_id'])
        self.assertEqual('p1', snet['subnet']['project_id'])
        self.assertEqual(4, snet['subnet']['ip_version'])
        self.assertEqual(net_id, snet['subnet']['network_id'])

    def test_create_port(self):
        mock_plugin = mock.Mock()
        mock_plugin.create_port = lambda c, p: p
        mock_ctx = mock.Mock()
        mock_ctx.project_id = 'p1'
        net_id = uuidutils.generate_uuid()
        port = utils.create_port(
            mock_plugin, mock_ctx,
            {'port': {'network_id': net_id, 'name': 'aport'}})
        self.assertEqual('p1', port['port']['tenant_id'])
        self.assertEqual('p1', port['port']['project_id'])
        self.assertEqual('aport', port['port']['name'])
        self.assertEqual(net_id, port['port']['network_id'])

    def test_get_port_binding_by_status_and_host(self):
        bindings = []
        self.assertIsNone(utils.get_port_binding_by_status_and_host(
                          bindings, constants.INACTIVE))
        bindings.extend([{pb_ext.STATUS: constants.INACTIVE,
                          pb_ext.HOST: 'host-1'},
                         {pb_ext.STATUS: constants.INACTIVE,
                          pb_ext.HOST: 'host-2'}])
        self.assertEqual(
            'host-1', utils.get_port_binding_by_status_and_host(
                bindings,
                constants.INACTIVE)[pb_ext.HOST])
        self.assertEqual(
            'host-2', utils.get_port_binding_by_status_and_host(
                bindings,
                constants.INACTIVE,
                host='host-2')[pb_ext.HOST])
        self.assertIsNone(utils.get_port_binding_by_status_and_host(
                          bindings, constants.ACTIVE))
        self.assertRaises(exceptions.PortBindingNotFound,
                          utils.get_port_binding_by_status_and_host, bindings,
                          constants.ACTIVE, 'host', True, 'port_id')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import firewall_v2
from neutron_lib import constants
from neutron_lib.tests.unit.api.definitions import base


class FirewallDefinitionTestCase(base.DefinitionBaseTestCase):
    extension_module = firewall_v2
    extension_resources = ('firewall_groups', 'firewall_policies',
                           'firewall_rules')
    extension_attributes = ('action', 'admin_state_up', 'audited',
                            'destination_ip_address', 'destination_port',
                            'egress_firewall_policy_id', 'enabled',
                            'firewall_policy_id', 'firewall_rules',
                            'ingress_firewall_policy_id', 'ip_version',
                            'ports', 'position', 'protocol', constants.SHARED,
                            'source_ip_address', 'source_port',
                            'source_firewall_group_id',
                            'destination_firewall_group_id')



# Copyright 2018 Ericsson
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import uuid

from neutron_lib.placement import utils as place_utils
from neutron_lib.tests import _base as base


class TestPlacementUtils(base.BaseTestCase):

    def setUp(self):
        super(TestPlacementUtils, self).setUp()

        self._uuid_ns = uuid.UUID('94fedd4d-1ce0-4bb3-9c9a-c9c0f56de154')

    def test_physnet_trait(self):
        self.assertEqual(
            'CUSTOM_PHYSNET_SOME_PHYSNET',
            place_utils.physnet_trait('some-physnet'))

    def test_vnic_type_trait(self):
        self.assertEqual(
            'CUSTOM_VNIC_TYPE_SOMEVNICTYPE',
            place_utils.vnic_type_trait('somevnictype'))

    def test_six_uuid5_literal(self):
        try:
            # assertNotRaises
            place_utils.six_uuid5(
                namespace=self._uuid_ns,
                name='may or may not be a unicode string' +
                     ' depending on Python version')
        except Exception:
            self.fail('could not generate uuid')

    def test_six_uuid5_unicode(self):
        try:
            # assertNotRaises
            place_utils.six_uuid5(
                namespace=self._uuid_ns,
                name=u'unicode string')
        except Exception:
            self.fail('could not generate uuid')

    def test_agent_resource_provider_uuid(self):
        try:
            # assertNotRaises
            place_utils.agent_resource_provider_uuid(
                namespace=self._uuid_ns,
                host='some host')
        except Exception:
            self.fail('could not generate agent resource provider uuid')

    def test_device_resource_provider_uuid(self):
        try:
            # assertNotRaises
            place_utils.device_resource_provider_uuid(
                namespace=self._uuid_ns,
                host='some host',
                device='some device')
        except Exception:
            self.fail('could not generate device resource provider uuid')

    def test_agent_resource_provider_uuid_stable(self):
        uuid_a = place_utils.agent_resource_provider_uuid(
            namespace=self._uuid_ns,
            host='somehost')
        uuid_b = place_utils.agent_resource_provider_uuid(
            namespace=self._uuid_ns,
            host='somehost')
        self.assertEqual(uuid_a, uuid_b)

    def test_device_resource_provider_uuid_stable(self):
        uuid_a = place_utils.device_resource_provider_uuid(
            namespace=self._uuid_ns,
            host='somehost',
            device='some-device')
        uuid_b = place_utils.device_resource_provider_uuid(
            namespace=self._uuid_ns,
            host='somehost',
            device='some-device')
        self.assertEqual(uuid_a, uuid_b)

    def test_parse_rp_bandwidths(self):
        self.assertEqual(
            {},
            place_utils.parse_rp_bandwidths([]),
        )

        self.assertEqual(
            {'eth0': {'egress': None, 'ingress': None}},
            place_utils.parse_rp_bandwidths(['eth0']),
        )

        self.assertEqual(
            {'eth0': {'egress': None, 'ingress': None}},
            place_utils.parse_rp_bandwidths(['eth0::']),
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_bandwidths,
            ['eth0::', 'eth0::'],
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_bandwidths,
            ['eth0:not a number:not a number'],
        )

        self.assertEqual(
            {'eth0': {'egress': 1, 'ingress': None}},
            place_utils.parse_rp_bandwidths(['eth0:1:']),
        )

        self.assertEqual(
            {'eth0': {'egress': None, 'ingress': 1}},
            place_utils.parse_rp_bandwidths(['eth0::1']),
        )

        self.assertEqual(
            {'eth0': {'egress': 1, 'ingress': 1}},
            place_utils.parse_rp_bandwidths(['eth0:1:1']),
        )

        self.assertEqual(
            {'eth0': {'egress': 1, 'ingress': 1},
             'eth1': {'egress': 10, 'ingress': 10}},
            place_utils.parse_rp_bandwidths(['eth0:1:1', 'eth1:10:10']),
        )

    def test_parse_rp_inventory_defaults(self):
        self.assertEqual(
            {},
            place_utils.parse_rp_inventory_defaults({}),
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_inventory_defaults,
            {'allocation_ratio': '-1.0'}
        )

        self.assertEqual(
            {'allocation_ratio': 1.0},
            place_utils.parse_rp_inventory_defaults(
                {'allocation_ratio': '1.0'}),
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_inventory_defaults,
            {'min_unit': '-1'}
        )

        self.assertEqual(
            {'min_unit': 1},
            place_utils.parse_rp_inventory_defaults(
                {'min_unit': '1'}),
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_inventory_defaults,
            {'no such inventory parameter': 1}
        )



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Well-known service type constants:
FIREWALL = "FIREWALL"
VPN = "VPN"
METERING = "METERING"
FLAVORS = "FLAVORS"
QOS = "QOS"
CORE = 'CORE'
L3 = 'L3_ROUTER_NAT'
LOG_API = "LOGGING"
PORTFORWARDING = "PORTFORWARDING"
FLOATINGIPPOOL = "FLOATINGIPPOOL"
NETWORK_SEGMENT_RANGE = "NETWORK_SEGMENT_RANGE"
CONNTRACKHELPER = "CONNTRACKHELPER"
PLACEMENT_REPORT = "placement_report"

# TODO(johnsom) Remove after these stop being used. Neutron-LBaaS is now
#               retired (train) and these should no longer be necessary.
LOADBALANCER = "LOADBALANCER"
LOADBALANCERV2 = "LOADBALANCERV2"



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutron_lib.api.definitions import metering

ALIAS = 'metering_source_and_destination_fields'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Neutron Metering with source and destination filtering'
API_PREFIX = ''
DESCRIPTION = 'Neutron Metering extension that enables the use of source ' \
              'and destination IP prefixes to create metering label rules.'
UPDATED_TIMESTAMP = '2020-08-21T08:12:00-00:00'

RESOURCE_ATTRIBUTE_MAP = {}

REQUIRED_EXTENSIONS = [metering.ALIAS]

_PARENT = {
    'collection_name': metering.METERING_LABEL_RULES,
    'member_name': metering.METERING_LABEL_RULES
}
SUB_RESOURCE_ATTRIBUTE_MAP = {
    metering.METERING_LABEL_RULES: {
        'parent': _PARENT,
        'parameters': {
            'remote_ip_prefix': {
                'allow_post': True, 'allow_put': False,
                'is_visible': True, 'required_by_policy': False,
                'is_filter': True, 'is_sort_key': True, 'default': None
            },
            'source_ip_prefix': {
                'allow_post': True, 'allow_put': True,
                'is_visible': True, 'required_by_policy': False,
                'is_filter': True, 'is_sort_key': True, 'default': None
            },
            'destination_ip_prefix': {
                'allow_post': True, 'allow_put': True,
                'is_visible': True, 'required_by_policy': False,
                'is_filter': True, 'is_sort_key': True, 'default': None
            }
        }
    }
}
ACTION_MAP = {}
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from stevedore import enabled

from neutron_lib.tests import _base as base
from neutron_lib.utils import runtime


class _DummyDriver(object):
    driver = mock.sentinel.dummy_driver


class TestRunTime(base.BaseTestCase):

    @mock.patch.object(runtime, 'LOG')
    def test_load_class_by_alias_or_classname_no_name(self, mock_log):
        self.assertRaises(
            ImportError,
            runtime.load_class_by_alias_or_classname, 'ns', None)

    @mock.patch.object(runtime.driver, 'DriverManager',
                       return_value=_DummyDriver)
    @mock.patch.object(runtime, 'LOG')
    def test_load_class_by_alias_or_classname_dummy_driver(
            self, mock_log, mock_driver):
        self.assertEqual(_DummyDriver.driver,
                         runtime.load_class_by_alias_or_classname('ns', 'n'))

    @mock.patch.object(runtime, 'LOG')
    def test_load_class_by_alias_or_classname_bad_classname(self, mock_log):
        self.assertRaises(
            ImportError,
            runtime.load_class_by_alias_or_classname, 'ns', '_NoClass')

    @mock.patch.object(runtime.importutils, 'import_class',
                       return_value=mock.sentinel.dummy_class)
    @mock.patch.object(runtime, 'LOG')
    def test_load_class_by_alias_or_classname_with_classname(
            self, mock_log, mock_import):
        self.assertEqual(
            mock.sentinel.dummy_class,
            runtime.load_class_by_alias_or_classname('ns', 'n'))


class TestNamespacedPlugins(base.BaseTestCase):

    @mock.patch.object(enabled, 'EnabledExtensionManager')
    def test_init_reload(self, mock_mgr):
        plugins = runtime.NamespacedPlugins('_test_ns_')
        mock_mgr.assert_called_with(
            '_test_ns_', mock.ANY, invoke_on_load=False)
        mock_mgr().map.assert_called_with(plugins._add_extension)

    @mock.patch.object(runtime, 'LOG')
    @mock.patch.object(enabled, 'EnabledExtensionManager')
    def test_init_reload_no_plugins(self, mock_mgr, mock_log):
        mock_mgr().names.return_value = []
        plugins = runtime.NamespacedPlugins('_test_ns_')
        mock_log.debug.assert_called_once()
        mock_mgr().map.assert_not_called()
        self.assertDictEqual({}, plugins._extensions)

    @mock.patch.object(enabled, 'EnabledExtensionManager')
    def test_add_duplicate_names(self, mock_mgr):
        mock_ep = mock.Mock()
        mock_ep.name = 'a'
        mock_mgr().names.return_value = ['a', 'a']
        # return 2 EPs with the same name
        mock_mgr().map = lambda f: [f(ep) for ep in [mock_ep, mock_ep]]
        self.assertRaises(KeyError, runtime.NamespacedPlugins, '_test_ns_')

    @mock.patch.object(enabled, 'EnabledExtensionManager')
    def test_get_plugin_class(self, mock_mgr):
        mock_epa = mock.Mock()
        mock_epa.name = 'a'
        mock_epa.plugin = 'A'
        mock_epb = mock.Mock()
        mock_epb.name = 'b'
        mock_epb.plugin = 'B'
        mock_mgr().names.return_value = ['a', 'b']
        mock_mgr().map = lambda f: [f(ep) for ep in [mock_epa, mock_epb]]

        plugins = runtime.NamespacedPlugins('_test_ns_')
        self.assertEqual('A', plugins.get_plugin_class('a'))
        self.assertEqual('B', plugins.get_plugin_class('b'))

    @mock.patch.object(enabled, 'EnabledExtensionManager')
    def test_new_plugin_instance(self, mock_mgr):
        mock_epa = mock.Mock()
        mock_epa.name = 'a'
        mock_epb = mock.Mock()
        mock_epb.name = 'b'
        mock_mgr().names.return_value = ['a', 'b']
        mock_mgr().map = lambda f: [f(ep) for ep in [mock_epa, mock_epb]]

        plugins = runtime.NamespacedPlugins('_test_ns_')
        plugins.new_plugin_instance('a', 'c', 'd', karg='kval')
        plugins.new_plugin_instance('b')
        mock_epa.plugin.assert_called_once_with('c', 'd', karg='kval')
        mock_epb.plugin.assert_called_once_with()


class TestListPackageModules(base.BaseTestCase):

    def test_list_package_modules(self):
        # mainly just to ensure we can import modules for both PY2/PY3
        self.assertGreater(
            len(runtime.list_package_modules('neutron_lib.exceptions')), 3)



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_utils import timeutils
import sqlalchemy as sa
from sqlalchemy import event  # noqa
from sqlalchemy.ext.associationproxy import association_proxy
from sqlalchemy.ext import declarative
from sqlalchemy.orm import attributes
from sqlalchemy.orm import session as se

from neutron_lib._i18n import _
from neutron_lib.db import constants as db_const
from neutron_lib.db import model_base
from neutron_lib.db import sqlalchemytypes


class StandardAttribute(model_base.BASEV2):
    """Common table to associate all Neutron API resources.

    By having Neutron objects related to this table, we can associate new
    tables that apply to many Neutron objects (e.g. timestamps, rbac entries)
    to this table to avoid schema duplication while maintaining referential
    integrity.

    NOTE(kevinbenton): This table should not have more columns added to it
    unless we are absolutely certain the new column will have a value for
    every single type of Neutron resource. Otherwise this table will be filled
    with NULL entries for combinations that don't make sense. Additionally,
    by keeping this table small we can ensure that performance isn't adversely
    impacted for queries on objects.
    """

    # sqlite doesn't support auto increment on big integers so we use big int
    # for everything but sqlite
    id = sa.Column(sa.BigInteger().with_variant(sa.Integer(), 'sqlite'),
                   primary_key=True, autoincrement=True)

    # NOTE(kevinbenton): this column is redundant information, but it allows
    # operators/devs to look at the contents of this table and know which table
    # the corresponding object is in.
    # 255 was selected as a max just because it's the varchar ceiling in mysql
    # before a 2-byte prefix is required. We shouldn't get anywhere near this
    # limit with our table names...
    resource_type = sa.Column(sa.String(255), nullable=False)
    description = sa.Column(sa.String(db_const.DESCRIPTION_FIELD_SIZE))

    revision_number = sa.Column(
        sa.BigInteger().with_variant(sa.Integer(), 'sqlite'),
        server_default='0', nullable=False)
    created_at = sa.Column(sqlalchemytypes.TruncatedDateTime,
                           default=timeutils.utcnow)
    updated_at = sa.Column(sqlalchemytypes.TruncatedDateTime,
                           onupdate=timeutils.utcnow)

    __mapper_args__ = {
        # see http://docs.sqlalchemy.org/en/latest/orm/versioning.html for
        # details about how this works
        "version_id_col": revision_number,
        "version_id_generator": False  # revision plugin increments manually
    }

    def bump_revision(self):
        if self.revision_number is None:
            # this is a brand new object uncommitted so we don't bump now
            return
        self.revision_number += 1

    def _set_updated_revision_number(self, revision_number, updated_at):
        attributes.set_committed_value(
            self, "revision_number", revision_number)
        attributes.set_committed_value(
            self, "updated_at", updated_at)

    @property
    def _effective_standard_attribute_id(self):
        return self.id


class HasStandardAttributes(object):

    @classmethod
    def get_api_collections(cls):
        """Define the API collection this object will appear under.

        This should return a list of API collections that the object
        will be exposed under. Most should be exposed in just one
        collection (e.g. the network model is just exposed under
        'networks').

        This is used by the standard attr extensions to discover which
        resources need to be extended with the standard attr fields
        (e.g. created_at/updated_at/etc).
        """
        # NOTE(kevinbenton): can't use abc because the metaclass conflicts
        # with the declarative base others inherit from.
        if hasattr(cls, 'api_collections'):
            return cls.api_collections
        raise NotImplementedError(_("%s must define api_collections") % cls)

    @classmethod
    def get_api_sub_resources(cls):
        """Define the API sub-resources this object will appear under.

        This should return a list of API sub-resources that the object
        will be exposed under.

        This is used by the standard attr extensions to discover which
        sub-resources need to be extended with the standard attr fields
        (e.g. created_at/updated_at/etc).
        """
        try:
            return cls.api_sub_resources
        except AttributeError:
            return []

    @classmethod
    def get_collection_resource_map(cls):
        try:
            return cls.collection_resource_map
        except AttributeError as e:
            raise NotImplementedError(
                _("%s must define collection_resource_map") % cls) from e

    @classmethod
    def validate_tag_support(cls):
        return getattr(cls, 'tag_support', False)

    @declarative.declared_attr
    def standard_attr_id(cls):
        return sa.Column(
            sa.BigInteger().with_variant(sa.Integer(), 'sqlite'),
            sa.ForeignKey(StandardAttribute.id, ondelete="CASCADE"),
            unique=True,
            nullable=False
        )

    # NOTE(kevinbenton): we have to disable the following pylint check because
    # it thinks we are overriding this method in the __init__ method.
    # pylint: disable=method-hidden
    @declarative.declared_attr
    def standard_attr(cls):
        return sa.orm.relationship(StandardAttribute,
                                   lazy='joined',
                                   cascade='all, delete-orphan',
                                   single_parent=True,
                                   uselist=False)

    @property
    def _effective_standard_attribute_id(self):
        return self.standard_attr_id

    def __init__(self, *args, **kwargs):
        standard_attr_keys = ['description', 'created_at',
                              'updated_at', 'revision_number']
        standard_attr_kwargs = {}
        for key in standard_attr_keys:
            if key in kwargs:
                standard_attr_kwargs[key] = kwargs.pop(key)
        super().__init__(*args, **kwargs)
        # here we automatically create the related standard attribute object
        self.standard_attr = StandardAttribute(
            resource_type=self.__tablename__, **standard_attr_kwargs)

    @declarative.declared_attr
    def description(cls):
        return association_proxy('standard_attr', 'description')

    @declarative.declared_attr
    def created_at(cls):
        return association_proxy('standard_attr', 'created_at')

    @declarative.declared_attr
    def updated_at(cls):
        return association_proxy('standard_attr', 'updated_at')

    def update(self, new_dict):
        # ignore the timestamps if they were passed in. For example, this
        # happens if code calls update_port with modified results of get_port
        new_dict.pop('created_at', None)
        new_dict.pop('updated_at', None)
        super().update(new_dict)

    @declarative.declared_attr
    def revision_number(cls):
        return association_proxy('standard_attr', 'revision_number')

    def bump_revision(self):
        # SQLAlchemy will bump the version for us automatically if the
        # standard attr record is being modified, but we must call this
        # for all other modifications or when relevant children are being
        # modified (e.g. fixed_ips change should bump port revision)
        self.standard_attr.bump_revision()

    def _set_updated_revision_number(self, revision_number, updated_at):
        self.standard_attr._set_updated_revision_number(
            revision_number, updated_at)


def _resource_model_map_helper(rs_map, resource, subclass):
    if resource in rs_map:
        raise RuntimeError(_("Model %(sub)s tried to register for API "
                             "resource %(res)s which conflicts with model "
                             "%(other)s.") %
                           dict(sub=subclass,
                                other=rs_map[resource],
                                res=resource))
    rs_map[resource] = subclass


def get_standard_attr_resource_model_map(include_resources=True,
                                         include_sub_resources=True):
    rs_map = {}
    for subclass in HasStandardAttributes.__subclasses__():
        if include_resources:
            for resource in subclass.get_api_collections():
                _resource_model_map_helper(rs_map, resource, subclass)
        if include_sub_resources:
            for sub_resource in subclass.get_api_sub_resources():
                _resource_model_map_helper(rs_map, sub_resource, subclass)
    return rs_map


def get_tag_resource_parent_map():
    parent_map = {}
    for subclass in HasStandardAttributes.__subclasses__():
        if subclass.validate_tag_support():
            for collection, resource in (subclass.get_collection_resource_map()
                                         .items()):
                if collection in parent_map:
                    msg = (_("API parent %(collection)s/%(resource)s for "
                             "model %(subclass)s is already registered.") %
                           dict(collection=collection, resource=resource,
                                subclass=subclass))
                    raise RuntimeError(msg)
                parent_map[collection] = resource
    return parent_map


@event.listens_for(se.Session, 'after_bulk_delete')
def throw_exception_on_bulk_delete_of_listened_for_objects(delete_context):
    if hasattr(delete_context.mapper.class_, 'revises_on_change'):
        raise RuntimeError(_("%s may not be deleted in bulk because it "
                             "bumps the revision of other resources via "
                             "SQLAlchemy event handlers, which are not "
                             "compatible with bulk deletes.") %
                           delete_context.mapper.class_)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import constants


KNOWN_ATTRIBUTES = (
    'admin_state_up',
    'allocation_pools',
    'cidr',
    'default_prefixlen',
    'default_quota',
    'description',
    'device_id',
    'device_owner',
    'dns_nameservers',
    'enable_dhcp',
    'fixed_ips',
    'gateway_ip',
    'host_routes',
    'id',
    'ip_version',
    'ipv6_address_mode',
    'ipv6_ra_mode',
    'is_default',
    'mac_address',
    'max_prefixlen',
    'min_prefixlen',
    'name',
    'network_id',
    'port_id',
    'prefixes',
    'prefixlen',
    'project_id',
    'qos_policy_id',
    'service_types',
    constants.SHARED,
    'status',
    'subnets',
    'subnetpool_id',
    'tenant_id'
)

KNOWN_RESOURCES = (
    'networks',
    'ports',
    'routers',
    'subnets',
    'subnetpools',
    'security_groups'
)

KNOWN_HTTP_ACTIONS = (
    'DELETE',
    'GET',
    'POST',
    'PUT',
)

KNOWN_ACTION_STATUSES = (
    200,
    201,
    202,
    203,
    204,
    205,
    206,
)

KNOWN_EXTENSIONS = (
    'address-scope',
    'agent',
    'agent-resources-synced',
    'allowed-address-pairs',
    'auto-allocated-topology',
    'availability_zone',
    'binding',
    'data-plane-status',
    'project-default-networks',
    'default-subnetpools',
    'dhcp_agent_scheduler',
    'dns-domain-ports',
    'dns-integration',
    'dvr',
    'empty-string-filtering',
    'expose-l3-conntrack-helper',
    'expose-port-forwarding-in-fip',
    'ext-gw-mode',
    'external-net',
    'extra_dhcp_opt',
    'extraroute',
    'extraroute-atomic',
    'filter-validation',
    'fip-port-details',
    'flavors',
    'floating-ip-port-forwarding',
    'floating-ip-port-forwarding-description',
    'floatingip-autodelete-internal',
    'floatingip-pools',
    'ip-substring-filtering',
    'l3-conntrack-helper',
    'l3-ha',
    'l3_agent_scheduler',
    'l3-port-ip-change-not-allowed',
    'logging',
    'metering',
    'metering_source_and_destination_filters',
    'multi-provider',
    'net-mtu',
    'network-ip-availability',
    'network-segment-range',
    'network_availability_zone',
    'pagination',
    'port-resource-request',
    'port-security',
    'project-id',
    'provider',
    'qos',
    'qos-bw-limit-direction',
    'qos-gateway-ip',
    'qos-port-network-policy',
    'qos-rules-alias',
    'quotas',
    'rbac-address-group',
    'rbac-address-scope',
    'rbac-policies',
    'rbac-security-groups',
    'rbac-subnetpool',
    'router',
    'router_availability_zone',
    'security-group',
    'segment',
    'service-type',
    'sort-key-validation',
    'sorting',
    'standard-attr-description',
    'standard-attr-revisions',
    'standard-attr-segment',
    'standard-attr-timestamp',
    'subnet',
    'subnet_allocation',
    'subnet_dns_publish_fixed_ip',
    'subnet_onboard',
    'subnetpool-prefix-ops',
    'subnet-segmentid-enforce',
    'subnet-segmentid-writable',
    'tag',
    'trunk',
    'trunk-details',
    # Add here list of extensions with pointers to the project repo, e.g.
    # 'bgp',  # https://opendev.org/openstack/neutron-dynamic-routing

    # https://opendev.org/openstack/neutron-fwaas
    'fwaas',
    'fwaasrouterinsertion',
    'fwaas_v2',
    'bgpvpn',  # https://opendev.org/openstack/networking-bgpvpn
    'bgpvpn-routes-control',
    'bgpvpn-vni',

    # https://opendev.org/openstack/neutron-vpnaas
    'vpnaas',
    'vpn-endpoint-groups',
    'vpn-flavors',

    # https://opendev.org/openstack/networking-sfc:
    'flow_classifier',
    'sfc',
)

KNOWN_KEYWORDS = (
    'allow_post',
    'allow_put',
    'convert_to',
    'convert_list_to',
    'default',
    'enforce_policy',
    'is_filter',
    'is_sort_key',
    'is_visible',
    'primary_key',
    'required_by_policy',
    'validate',
    'default_overrides_none',
    'dict_populate_defaults',
)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from neutron_lib.agent import extension


class L2AgentExtension(extension.AgentExtension, metaclass=abc.ABCMeta):
    """Define stable abstract interface for l2 agent extensions.

    An agent extension extends the agent core functionality.
    """

    def initialize(self, connection, driver_type):
        """Initialize agent extension.

        :param connection: RPC connection that can be reused by the extension
                           to define its RPC endpoints
        :param driver_type: String that defines the agent type to the
                            extension. Can be used to choose the right backend
                            implementation.
        """

    @abc.abstractmethod
    def handle_port(self, context, data):
        """Handle a port add/update event.

        This can be called on either create or update, depending on the
        code flow. Thus, it's this function's responsibility to check what
        actually changed.

        :param context: RPC context.
        :param data: Port data.
        """

    @abc.abstractmethod
    def delete_port(self, context, data):
        """Handle a port delete event.

        :param context: RPC context.
        :param data: Port data.
        """



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_db.sqlalchemy import models
from oslo_utils import uuidutils
import sqlalchemy as sa
from sqlalchemy.ext import declarative
from sqlalchemy import orm

from neutron_lib.db import constants as db_const


class HasProject(object):
    """Project mixin, add to subclasses that have a user."""

    # NOTE: project_id is just a free form string
    project_id = sa.Column(sa.String(db_const.PROJECT_ID_FIELD_SIZE),
                           index=True)

    def get_tenant_id(self):
        return self.project_id

    def set_tenant_id(self, value):
        self.project_id = value

    @declarative.declared_attr
    def tenant_id(cls):
        return orm.synonym(
            'project_id',
            descriptor=property(cls.get_tenant_id, cls.set_tenant_id))


class HasProjectNoIndex(HasProject):
    """Project mixin, add to subclasses that have a user."""

    # NOTE: project_id is just a free form string
    project_id = sa.Column(sa.String(db_const.PROJECT_ID_FIELD_SIZE))


class HasProjectPrimaryKeyIndex(HasProject):
    """Project mixin, add to subclasses that have a user."""

    # NOTE: project_id is just a free form string
    project_id = sa.Column(sa.String(db_const.PROJECT_ID_FIELD_SIZE),
                           nullable=False, primary_key=True, index=True)


class HasProjectPrimaryKey(HasProject):
    """Project mixin, add to subclasses that have a user."""

    # NOTE: project_id is just a free form string
    project_id = sa.Column(sa.String(db_const.PROJECT_ID_FIELD_SIZE),
                           nullable=False, primary_key=True)


class HasId(object):
    """id mixin, add to subclasses that have an id."""

    id = sa.Column(sa.String(db_const.UUID_FIELD_SIZE),
                   primary_key=True,
                   default=uuidutils.generate_uuid)


class HasStatusDescription(object):
    """Status with description mixin."""

    status = sa.Column(sa.String(db_const.STATUS_FIELD_SIZE),
                       nullable=False)
    status_description = sa.Column(sa.String(db_const.DESCRIPTION_FIELD_SIZE))


class _NeutronBase(models.ModelBase):
    """Base class for Neutron Models."""

    __table_args__ = {'mysql_engine': 'InnoDB'}

    def __iter__(self):
        self._i = iter(orm.object_mapper(self).columns)
        return self

    def next(self):
        n = next(self._i).name
        return n, getattr(self, n)

    __next__ = next

    def __repr__(self):
        """sqlalchemy based automatic __repr__ method."""
        items = ['%s=%r' % (col.name, getattr(self, col.name))
                 for col in self.__table__.columns]
        return "<%s.%s[object at %x] {%s}>" % (self.__class__.__module__,
                                               self.__class__.__name__,
                                               id(self), ', '.join(items))


class NeutronBaseV2(_NeutronBase):

    @declarative.declared_attr
    def __tablename__(cls):
        # Use the pluralized name of the class as the table name.
        return cls.__name__.lower() + 's'


BASEV2 = declarative.declarative_base(cls=NeutronBaseV2)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import weakref

from oslo_concurrency import lockutils

from neutron_lib.plugins import constants


_synchronized = lockutils.synchronized_with_prefix("neutron-")


class _PluginDirectory(object):
    """A directory of activated plugins in a Neutron Deployment.

    The directory is bootstrapped by a Neutron Manager running in
    the context of a Neutron Server process.
    """

    def __init__(self):
        self._plugins = {}

    def add_plugin(self, alias, plugin):
        """Add a plugin of type 'alias'."""
        self._plugins[alias] = plugin

    def get_plugin(self, alias):
        """Get a plugin for a given alias or None if not present."""
        p = self._plugins.get(alias)
        return weakref.proxy(p) if p else None

    @property
    def plugins(self):
        """The mapping alias -> weak reference to the plugin."""
        return dict((x, weakref.proxy(y))
                    for x, y in self._plugins.items())

    @property
    def unique_plugins(self):
        """A sequence of the unique plugins activated in the environments."""
        return tuple(weakref.proxy(x) for x in set(self._plugins.values()))

    @property
    def is_loaded(self):
        """True if the directory is non empty."""
        return len(self._plugins) > 0


# Create a singleton plugins directory for the Neutron server instance.
# Accessing these methods before a Neutron Manager has had the chance
# to load the environment may result in callers handling an empty directory.
_PLUGIN_DIRECTORY = None


@_synchronized("plugin-directory")
def _create_plugin_directory():
    global _PLUGIN_DIRECTORY
    if _PLUGIN_DIRECTORY is None:
        _PLUGIN_DIRECTORY = _PluginDirectory()
    return _PLUGIN_DIRECTORY


def _get_plugin_directory():
    if _PLUGIN_DIRECTORY is None:
        return _create_plugin_directory()
    return _PLUGIN_DIRECTORY


def add_plugin(alias, plugin):
    _get_plugin_directory().add_plugin(alias, plugin)


def get_plugin(alias=constants.CORE):
    return _get_plugin_directory().get_plugin(alias)


def get_plugins():
    return _get_plugin_directory().plugins


def get_unique_plugins():
    return _get_plugin_directory().unique_plugins


def is_loaded():
    return _get_plugin_directory().is_loaded



# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib.db import constants

# The alias of the extension.
ALIAS = 'router'

# Whether or not this extension is simply signaling behavior to the user
# or it actively modifies the attribute map.
IS_SHIM_EXTENSION = False

# Whether the extension is marking the adoption of standardattr model for
# legacy resources, or introducing new standardattr attributes. False or
# None if the standardattr model is adopted since the introduction of
# resource extension.
# If this is True, the alias for the extension should be prefixed with
# 'standard-attr-'.
IS_STANDARD_ATTR_EXTENSION = False

# The name of the extension.
NAME = 'Neutron L3 Router'

# A prefix for API resources. An empty prefix means that the API is going
# to be exposed at the v2/ level as any other core resource.
API_PREFIX = ''

# The description of the extension.
DESCRIPTION = ("Router abstraction for basic L3 forwarding "
               "between L2 Neutron networks and access to external "
               "networks via a NAT gateway.")

# A timestamp of when the extension was introduced.
UPDATED_TIMESTAMP = "2012-07-20T10:00:00-00:00"


FLOATING_IP_ADDRESS = 'floating_ip_address'
FLOATING_NETWORK_ID = 'floating_network_id'
FLOATINGIP = 'floatingip'
FLOATINGIPS = 'floatingips'
ROUTERS = 'routers'
ROUTER = 'router'
SUBNET_ID = 'subnet_id'
ROUTER_ID = 'router_id'
PORT_ID = 'port_id'
FIXED_IP_ADDRESS = 'fixed_ip_address'
EXTERNAL_GW_INFO = 'external_gateway_info'


RESOURCE_ATTRIBUTE_MAP = {
    ROUTERS: {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:uuid': None},
               'is_visible': True,
               'is_filter': True,
               'is_sort_key': True,
               'primary_key': True},
        'name': {'allow_post': True, 'allow_put': True,
                 'validate': {'type:string': constants.NAME_FIELD_SIZE},
                 'is_filter': True,
                 'is_sort_key': True,
                 'is_visible': True, 'default': ''},
        'admin_state_up': {'allow_post': True, 'allow_put': True,
                           'default': True,
                           'convert_to': converters.convert_to_boolean,
                           'is_filter': True,
                           'is_sort_key': True,
                           'is_visible': True},
        'status': {'allow_post': False, 'allow_put': False,
                   'is_sort_key': True,
                   'is_visible': True},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'required_by_policy': True,
                      'validate': {
                          'type:string': constants.PROJECT_ID_FIELD_SIZE},
                      'is_filter': True,
                      'is_sort_key': True,
                      'is_visible': True},
        EXTERNAL_GW_INFO: {'allow_post': True, 'allow_put': True,
                           'is_visible': True, 'default': None,
                           'enforce_policy': True,
                           'validate': {
                               'type:dict_or_nodata': {
                                   'network_id': {'type:uuid': None,
                                                  'required': True},
                                   'external_fixed_ips': {
                                       'type:fixed_ips': None,
                                       'required': False,
                                   }
                               }
                           }}
    },
    FLOATINGIPS: {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:uuid': None},
               'is_visible': True,
               'is_filter': True,
               'is_sort_key': True,
               'primary_key': True},
        'floating_ip_address': {'allow_post': True, 'allow_put': False,
                                'validate': {'type:ip_address_or_none': None},
                                'is_sort_key': True, 'is_filter': True,
                                'is_visible': True, 'default': None,
                                'enforce_policy': True},
        'subnet_id': {'allow_post': True, 'allow_put': False,
                      'validate': {'type:uuid_or_none': None},
                      'is_visible': False,  # Use False for input only attr
                      'default': None},
        'floating_network_id': {'allow_post': True, 'allow_put': False,
                                'validate': {'type:uuid': None},
                                'is_filter': True, 'is_sort_key': True,
                                'is_visible': True},
        'router_id': {'allow_post': False, 'allow_put': False,
                      'validate': {'type:uuid_or_none': None},
                      'is_filter': True, 'is_sort_key': True,
                      'is_visible': True, 'default': None},
        'port_id': {'allow_post': True, 'allow_put': True,
                    'validate': {'type:uuid_or_none': None},
                    'is_filter': True,
                    'is_visible': True, 'default': None,
                    'required_by_policy': True},
        'fixed_ip_address': {'allow_post': True, 'allow_put': True,
                             'validate': {'type:ip_address_or_none': None},
                             'is_filter': True, 'is_sort_key': True,
                             'is_visible': True, 'default': None},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'required_by_policy': True,
                      'validate': {
                          'type:string': constants.PROJECT_ID_FIELD_SIZE},
                      'is_filter': True, 'is_sort_key': True,
                      'is_visible': True},
        'status': {'allow_post': False, 'allow_put': False,
                   'is_filter': True, 'is_sort_key': True,
                   'is_visible': True},
    },
}

# The subresource attribute map for the extension. It adds child resources
# to main extension's resource. The subresource map must have a parent and
# a parameters entry. If an extension does not need such a map, None can
# be specified (mandatory).
SUB_RESOURCE_ATTRIBUTE_MAP = None

# The action map: it associates verbs with methods to be performed on
# the API resource.
ACTION_MAP = {
    ROUTER: {
        'add_router_interface': 'PUT',
        'remove_router_interface': 'PUT'
    }
}

# The action status.
ACTION_STATUS = {
}

# The list of required extensions.
REQUIRED_EXTENSIONS = [
]

# The list of optional extensions.
OPTIONAL_EXTENSIONS = [
]



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pkgutil
import sys

from oslo_concurrency import lockutils
from oslo_log import log as logging
from oslo_utils import importutils
from stevedore import driver
from stevedore import enabled

from neutron_lib._i18n import _


LOG = logging.getLogger(__name__)
SYNCHRONIZED_PREFIX = 'neutron-'

# common synchronization decorator
synchronized = lockutils.synchronized_with_prefix(SYNCHRONIZED_PREFIX)


class NamespacedPlugins(object):
    """Wraps a stevedore plugin namespace to load/access its plugins."""

    def __init__(self, namespace):
        self.namespace = namespace
        self._extension_manager = None
        self._extensions = {}
        self.reload()

    def reload(self):
        """Force a reload of the plugins for this instances namespace.

        :returns: None.
        """
        self._extensions = {}
        self._extension_manager = enabled.EnabledExtensionManager(
            self.namespace, lambda x: True,
            invoke_on_load=False)

        if not self._extension_manager.names():
            LOG.debug("No plugins found in namespace: ", self.namespace)
            return
        self._extension_manager.map(self._add_extension)

    def _add_extension(self, ext):
        if ext.name in self._extensions:
            msg = _("Plugin '%(p)s' already in namespace: %(ns)s") % {
                'p': ext.name, 'ns': self.namespace}
            raise KeyError(msg)

        LOG.debug("Loaded plugin '%s' from namespace: %s",
                  ext.name, self.namespace)
        self._extensions[ext.name] = ext

    def _assert_plugin_loaded(self, plugin_name):
        if plugin_name not in self._extensions:
            msg = _("Plugin '%(p)s' not in namespace: %(ns)s") % {
                'p': plugin_name, 'ns': self.namespace}
            raise KeyError(msg)

    def get_plugin_class(self, plugin_name):
        """Gets a reference to a loaded plugin's class.

        :param plugin_name: The name of the plugin to get the class for.
        :returns: A reference to the loaded plugin's class.
        :raises: KeyError if plugin_name is not loaded.
        """
        self._assert_plugin_loaded(plugin_name)
        return self._extensions[plugin_name].plugin

    def new_plugin_instance(self, plugin_name, *args, **kwargs):
        """Create a new instance of a plugin.

        :param plugin_name: The name of the plugin to instantiate.
        :param args: Any args to pass onto the constructor.
        :param kwargs: Any kwargs to pass onto the constructor.
        :returns: A new instance of plugin_name.
        :raises: KeyError if plugin_name is not loaded.
        """
        self._assert_plugin_loaded(plugin_name)
        return self.get_plugin_class(plugin_name)(*args, **kwargs)

    @property
    def loaded_plugin_names(self):
        return self._extensions.keys()


def load_class_by_alias_or_classname(namespace, name):
    """Load a class using stevedore alias or the class name.

    :param namespace: The namespace where the alias is defined.
    :param name: The alias or class name of the class to be loaded.
    :returns: Class if it can be loaded.
    :raises ImportError: if class cannot be loaded.
    """
    if not name:
        LOG.error("Alias or class name is not set")
        raise ImportError(_("Class not found."))
    try:
        # Try to resolve class by alias
        mgr = driver.DriverManager(
            namespace, name, warn_on_missing_entrypoint=False)
        class_to_load = mgr.driver
    except RuntimeError:
        e1_info = sys.exc_info()
        # Fallback to class name
        try:
            class_to_load = importutils.import_class(name)
        except (ImportError, ValueError) as e:
            LOG.error("Error loading class by alias",
                      exc_info=e1_info)
            LOG.error("Error loading class by class name",
                      exc_info=True)
            raise ImportError(_("Class not found.")) from e
    return class_to_load


def list_package_modules(package_name):
    """Get a list of the modules for a given package.

    :param package_name: The package name to get modules for.
    :returns: A list of module objects for the said package name.
    """
    pkg_mod = importutils.import_module(package_name)
    modules = [pkg_mod]

    for mod in pkgutil.walk_packages(pkg_mod.__path__):
        _, mod_name, _ = mod
        fq_name = pkg_mod.__name__ + "." + mod_name
        modules.append(importutils.import_module(fq_name))

    return modules



# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import port
from neutron_lib.api.definitions import portbindings
from neutron_lib import constants
from neutron_lib.db import constants as db_const

# Plural of the resource
COLLECTION_NAME = 'bindings'

# Name of the resource
RESOURCE_NAME = 'binding'

# Parent
PARENT_RESOURCE_NAME = port.RESOURCE_NAME
PARENT_COLLECTION_NAME = port.COLLECTION_NAME

# Whether or not this extension is simply signaling behavior to the user
# or it actively modifies the attribute map.
IS_SHIM_EXTENSION = False

# Whether the extension is marking the adoption of standardattr model for
# legacy resources, or introducing new standardattr attributes. False or
# None if the standardattr model is adopted since the introduction of
# resource extension.
# If this is True, the alias for the extension should be prefixed with
# 'standard-attr-'.
IS_STANDARD_ATTR_EXTENSION = False

# Name of the extension
NAME = "Port Bindings Extended"

# Extension alias
ALIAS = 'binding-extended'

# Extension description
DESCRIPTION = "Expose port bindings of a virtual port to external application"

# Action name for activating a port binding
ACTIVATE_BINDING = 'activate'

# Timestamp of last update of this extension
UPDATED_TIMESTAMP = "2017-07-17T10:00:00-00:00"

# Extension attributes
HOST = 'host'
VIF_TYPE = 'vif_type'
VIF_DETAILS = 'vif_details'
VNIC_TYPE = 'vnic_type'
PROFILE = 'profile'
STATUS = 'status'
PROJECT_ID = 'project_id'

# No attribute map, this extension defines only sub resource
RESOURCE_ATTRIBUTE_MAP = {}

# Attribute map of bindings
SUB_RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {
        'parent': {
            'collection_name': PARENT_COLLECTION_NAME,
            'member_name': PARENT_RESOURCE_NAME},
        'parameters': {
            HOST: {'allow_post': True, 'allow_put': True,
                   'is_visible': True, 'primary_key': True,
                   'is_filter': True},
            VIF_TYPE: {'allow_post': False, 'allow_put': False,
                       'default': constants.ATTR_NOT_SPECIFIED,
                       'enforce_policy': True,
                       'is_visible': True, 'is_filter': True},
            VIF_DETAILS: {'allow_post': False, 'allow_put': False,
                          'default': constants.ATTR_NOT_SPECIFIED,
                          'enforce_policy': True,
                          'is_visible': True},
            VNIC_TYPE: {'allow_post': True, 'allow_put': True,
                        'default': portbindings.VNIC_NORMAL,
                        'is_visible': True,
                        'is_filter': True,
                        'validate': {'type:values': portbindings.VNIC_TYPES},
                        'enforce_policy': True},
            PROFILE: {'allow_post': True, 'allow_put': True,
                      'default': constants.ATTR_NOT_SPECIFIED,
                      'enforce_policy': True,
                      'validate': {'type:dict_or_none': None},
                      'is_visible': True},
            STATUS: {'allow_post': False, 'allow_put': False,
                     'is_visible': True, 'is_filter': True},
            PROJECT_ID: {'allow_post': True, 'allow_put': False,
                         'required_by_policy': True,
                         'validate': {
                             'type:string': db_const.PROJECT_ID_FIELD_SIZE},
                         'is_visible': True},
        }
    }
}

# activate is an action
ACTION_MAP = {
    RESOURCE_NAME: {
        ACTIVATE_BINDING: 'PUT',
    },
}

# No action statuses are required
ACTION_STATUS = {
}

# The list of required extensions
REQUIRED_EXTENSIONS = [
    portbindings.ALIAS,
]

# The list of optional extensions
OPTIONAL_EXTENSIONS = []



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutron_lib.db import model_query
from neutron_lib import fixture
from neutron_lib.tests import _base
from neutron_lib.utils import helpers


# TODO(boden): find a way to test other model_query functions

class TestHooks(_base.BaseTestCase):

    def setUp(self):
        super(TestHooks, self).setUp()
        self.useFixture(fixture.DBQueryHooksFixture())

    def _mock_hook(self, x):
        return x

    def test_register_hook(self):
        mock_model = mock.Mock()
        model_query.register_hook(
            mock_model, 'hook1', self._mock_hook,
            self._mock_hook, result_filters=self._mock_hook)
        self.assertEqual(1, len(model_query._model_query_hooks.keys()))
        hook_ref = helpers.make_weak_ref(self._mock_hook)
        registered_hooks = model_query.get_hooks(mock_model)
        self.assertEqual(1, len(registered_hooks))
        for d in registered_hooks:
            for k in d.keys():
                self.assertEqual(hook_ref, d.get(k))

    def test_register_hook_non_callables(self):
        mock_model = mock.Mock()
        model_query.register_hook(
            mock_model, 'hook1', self._mock_hook, {}, result_filters={})
        self.assertEqual(1, len(model_query._model_query_hooks.keys()))
        hook_ref = helpers.make_weak_ref(self._mock_hook)
        registered_hooks = model_query.get_hooks(mock_model)
        self.assertEqual(1, len(registered_hooks))
        for d in registered_hooks:
            for k in d.keys():
                if k == 'query':
                    self.assertEqual(hook_ref, d.get(k))
                else:
                    self.assertEqual({}, d.get(k))

    def test_get_values(self):
        mock_model = mock.Mock()
        mock_context = mock.Mock()
        with mock.patch.object(
                model_query, 'query_with_hooks') as query_with_hooks:
            query_with_hooks.return_value = [['value1'], ['value2']]
            values = model_query.get_values(mock_context, mock_model,
                                            'fake_field')
        self.assertEqual(['value1', 'value2'], values)
        query_with_hooks.assert_called_with(
            mock_context, mock_model, field='fake_field')



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron_lib.api.definitions import availability_zone as az
from neutron_lib.api.definitions import l3


ALIAS = 'router_availability_zone'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Router Availability Zone'
API_PREFIX = ''
DESCRIPTION = 'Availability zone support for router.'
UPDATED_TIMESTAMP = '2015-01-01T10:00:00-00:00'
RESOURCE_NAME = l3.ROUTER
COLLECTION_NAME = l3.ROUTERS
RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {
        az.COLLECTION_NAME: {
            'allow_post': False, 'allow_put': False,
            'is_visible': True
        },
        az.AZ_HINTS: {
            'allow_post': True, 'allow_put': False, 'is_visible': True,
            'validate': {'type:availability_zone_hint_list': None},
            'default': []
        }
    }
}
SUB_RESOURCE_ATTRIBUTE_MAP = {}
ACTION_MAP = {}
REQUIRED_EXTENSIONS = [l3.ALIAS, az.ALIAS]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



#    (c) Copyright 2015 NEC Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import vpn
from neutron_lib.db import constants as db_const

# VPN Endpoint type constants
VPN_ENDPOINT_TYPE_CIDR = 'cidr'
VPN_ENDPOINT_TYPE_NETWORK = 'network'
VPN_ENDPOINT_TYPE_ROUTER = 'router'
VPN_ENDPOINT_TYPE_SUBNET = 'subnet'
VPN_ENDPOINT_TYPE_VLAN = 'vlan'

VPN_SUPPORTED_ENDPOINT_TYPES = [
    VPN_ENDPOINT_TYPE_CIDR, VPN_ENDPOINT_TYPE_NETWORK,
    VPN_ENDPOINT_TYPE_ROUTER, VPN_ENDPOINT_TYPE_SUBNET, VPN_ENDPOINT_TYPE_VLAN,
]

ALIAS = 'vpn-endpoint-groups'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'VPN Endpoint Groups'
DESCRIPTION = 'VPN endpoint groups support'
UPDATED_TIMESTAMP = '2015-08-04T10:00:00-00:00'
API_PREFIX = '/vpn'
RESOURCE_ATTRIBUTE_MAP = {
    'endpoint_groups': {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:uuid': None},
               'is_visible': True,
               'primary_key': True},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'validate': {
                          'type:string': db_const.PROJECT_ID_FIELD_SIZE},
                      'required_by_policy': True,
                      'is_visible': True},
        'name': {'allow_post': True, 'allow_put': True,
                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                 'is_visible': True, 'default': ''},
        'description': {'allow_post': True, 'allow_put': True,
                        'validate': {
                            'type:string': db_const.DESCRIPTION_FIELD_SIZE},
                        'is_visible': True, 'default': ''},
        'type': {'allow_post': True, 'allow_put': False,
                 'validate': {'type:values': VPN_SUPPORTED_ENDPOINT_TYPES},
                 'is_visible': True},
        'endpoints': {'allow_post': True, 'allow_put': False,
                      'convert_to': converters.convert_to_list,
                      'is_visible': True,
                      'default': []},
    },
}
SUB_RESOURCE_ATTRIBUTE_MAP = {}
ACTION_MAP = {}
REQUIRED_EXTENSIONS = [vpn.ALIAS]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# Copyright 2013, Nachi Ueno, NTT MCL, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib._i18n import _
from neutron_lib import exceptions


class InvalidRoutes(exceptions.InvalidInput):
    message = _("Invalid format for routes: %(routes)s, %(reason)s")


class RouterInterfaceInUseByRoute(exceptions.InUse):
    message = _("Router interface for subnet %(subnet_id)s on router "
                "%(router_id)s cannot be deleted, as it is required "
                "by one or more routes.")


class RoutesExhausted(exceptions.BadRequest):
    message = _("Unable to complete operation for %(router_id)s. "
                "The number of routes exceeds the maximum %(quota)s.")



# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import firewall_v2
from neutron_lib.db import constants as db_const


# The alias of the extension.
ALIAS = 'logging-resource'

# Whether or not this extension is simply signaling behavior to the user
# or it actively modifies the attribute map.
IS_SHIM_EXTENSION = False

# Whether the extension is marking the adoption of standardattr model for
# legacy resources, or introducing new standardattr attributes. False or
# None if the standardattr model is adopted since the introduction of
# resource extension.
# If this is True, the alias for the extension should be prefixed with
# 'standard-attr-'.
IS_STANDARD_ATTR_EXTENSION = False

# The name of the extension.
NAME = 'Logging Resource Extension'

# A prefix for API resources. An empty prefix means that the API is going
# to be exposed at the v2/ level as any other core resource.
API_PREFIX = '/logging'

# The description of the extension.
DESCRIPTION = "The logging resource extension."

# A timestamp of when the extension was introduced.
UPDATED_TIMESTAMP = "2016-06-06T10:00:00-00:00"

# The name of the resource.
RESOURCE_NAME = 'logging_resource'

# The plural for the resource.
COLLECTION_NAME = 'logging_resources'

# Attributes
ENABLED = 'enabled'

# Sub resource
FIREWALL_LOGS = 'firewall_logs'


LOGGING_RESOURCE_ID = 'logging_resource_id'
FW_EVENT = 'fw_event'
FIREWALL_ID = 'firewall_id'


FW_EVENT_ACCEPT = 'ACCEPT'
FW_EVENT_DROP = 'DROP'
FW_EVENT_ALL = 'ALL'
FW_EVENTS = [FW_EVENT_ACCEPT, FW_EVENT_DROP, FW_EVENT_ALL]
LOG_COMMON_FIELDS = {
    'id': {'allow_post': False, 'allow_put': False,
           'validate': {'type:uuid': None},
           'is_visible': True, 'primary_key': True},
    'tenant_id': {'allow_post': True, 'allow_put': False,
                  'required_by_policy': True, 'is_visible': True},
    LOGGING_RESOURCE_ID: {'allow_post': False, 'allow_put': False,
                          'is_visible': True}
}

RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:uuid': None}, 'is_visible': True,
               'primary_key': True},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'required_by_policy': True, 'is_visible': True},
        'name': {'allow_post': True, 'allow_put': True,
                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                 'default': '', 'is_visible': True},
        'description': {
            'allow_post': True, 'allow_put': True,
            'validate': {'type:string': db_const.LONG_DESCRIPTION_FIELD_SIZE},
            'default': '', 'is_visible': True},
        ENABLED: {'allow_post': True, 'allow_put': True,
                  'is_visible': True, 'default': False,
                  'convert_to': converters.convert_to_boolean},
        FIREWALL_LOGS: {'allow_post': False, 'allow_put': False,
                        'is_visible': True}
    }
}


SUB_RESOURCE_ATTRIBUTE_MAP = {
    FIREWALL_LOGS: {
        'parent': {'collection_name': COLLECTION_NAME,
                   'member_name': RESOURCE_NAME},
        'parameters': dict((LOG_COMMON_FIELDS), **{
            'description': {
                'allow_post': True, 'allow_put': True,
                'validate': {
                    'type:string': db_const.LONG_DESCRIPTION_FIELD_SIZE},
                'default': '', 'is_visible': True},
            FIREWALL_ID: {
                'allow_post': True, 'allow_put': False,
                'is_visible': True,
                'validate': {'type:uuid': None}},
            FW_EVENT: {
                'allow_post': True, 'allow_put': True,
                'is_visible': True,
                'validate': {'type:values': FW_EVENTS},
                'default': FW_EVENT_ALL}
        }),
    },
}

# The action map.
ACTION_MAP = {}

# The action status.
ACTION_STATUS = {
}

# The list of required extensions.
REQUIRED_EXTENSIONS = [
    firewall_v2.ALIAS,
]

# The list of optional extensions.
OPTIONAL_EXTENSIONS = []



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import dvr as dvr_apidef

ALIAS = 'router-admin-state-down-before-update'
IS_SHIM_EXTENSION = True
IS_STANDARD_ATTR_EXTENSION = False
NAME = "Enforce Router's Admin State Down Before Update Extension"
DESCRIPTION = ('Ensure that the admin state of a router is DOWN '
               '(admin_state_up=False) before updating the distributed '
               'attribute')
UPDATED_TIMESTAMP = '2019-07-02T15:56:00'
RESOURCE_ATTRIBUTE_MAP = {}
SUB_RESOURCE_ATTRIBUTE_MAP = {}
ACTION_MAP = {}
REQUIRED_EXTENSIONS = [dvr_apidef.ALIAS]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib._i18n import _
from neutron_lib import exceptions


class FlavorNotFound(exceptions.NotFound):
    message = _("Flavor %(flavor_id)s could not be found.")


class FlavorInUse(exceptions.InUse):
    message = _("Flavor %(flavor_id)s is used by some service instance.")


class ServiceProfileNotFound(exceptions.NotFound):
    message = _("Service Profile %(sp_id)s could not be found.")


class ServiceProfileInUse(exceptions.InUse):
    message = _("Service Profile %(sp_id)s is used by some service instance.")


class FlavorServiceProfileBindingExists(exceptions.Conflict):
    message = _("Service Profile %(sp_id)s is already associated "
                "with flavor %(fl_id)s.")


class FlavorServiceProfileBindingNotFound(exceptions.NotFound):
    message = _("Service Profile %(sp_id)s is not associated "
                "with flavor %(fl_id)s.")


class ServiceProfileDriverNotFound(exceptions.NotFound):
    message = _("Service Profile driver %(driver)s could not be found.")


class ServiceProfileEmpty(exceptions.InvalidInput):
    message = _("Service Profile needs either a driver or metainfo.")


class FlavorDisabled(exceptions.ServiceUnavailable):
    message = _("Flavor is not enabled.")


class ServiceProfileDisabled(exceptions.ServiceUnavailable):
    message = _("Service Profile is not enabled.")



# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import l3
from neutron_lib.db import constants as db_const

# Regular expression to validate 32 bits unsigned int
UINT32_REGEX = (r'(0|[1-9]\d{0,8}|[1-3]\d{9}|4[01]\d{8}|42[0-8]\d{7}'
                r'|429[0-3]\d{6}|4294[0-8]\d{5}|42949[0-5]\d{4}'
                r'|429496[0-6]\d{3}|4294967[0-1]\d{2}|42949672[0-8]\d'
                r'|429496729[0-5])')
# Regular expression  to validate 16 bits unsigned int
UINT16_REGEX = (r'(0|[1-9]\d{0,3}|[1-5]\d{4}|6[0-4]\d{3}|65[0-4]\d{2}'
                r'|655[0-2]\d|6553[0-5])')
# Regular expression to validate 8 bits unsigned int
UINT8_REGEX = (r'(0|[1-9]\d{0,1}|1\d{2}|2[0-4]\d|25[0-5])')
# Regular expression to validate IPv4 address
IP4_REGEX = (r'(%s\.%s\.%s\.%s)') % (UINT8_REGEX, UINT8_REGEX, UINT8_REGEX,
                                     UINT8_REGEX)
# Regular expression to validate Route Target list format
# Support of the Type 0, Type 1 and Type 2, cf. chapter 4.2 in RFC 4364
# Also validates Route Distinguisher list format
RTRD_REGEX = (r'^(%s:%s|%s:%s|%s:%s)$') % (UINT16_REGEX, UINT32_REGEX,
                                           IP4_REGEX, UINT16_REGEX,
                                           UINT32_REGEX, UINT16_REGEX)

# The alias of the extension.
ALIAS = 'bgpvpn'

# Whether or not this extension is simply signaling behavior to the user
# or it actively modifies the attribute map.
IS_SHIM_EXTENSION = False

# Whether the extension is marking the adoption of standardattr model for
# legacy resources, or introducing new standardattr attributes. False or
# None if the standardattr model is adopted since the introduction of
# resource extension.
# If this is True, the alias for the extension should be prefixed with
# 'standard-attr-'.
IS_STANDARD_ATTR_EXTENSION = False

# The name of the extension.
NAME = 'BGPVPN Extension'

# The description of the extension.
DESCRIPTION = "Provides support for BGP VPN interconnections"

# A timestamp of when the extension was introduced.
UPDATED_TIMESTAMP = "2014-06-10T17:00:00-00:00"

API_PREFIX = '/bgpvpn'


# The specific resources and/or attributes for the extension (optional).
RESOURCE_NAME = 'bgpvpn'
COLLECTION_NAME = 'bgpvpns'
BGPVPN_L2 = 'l2'
BGPVPN_L3 = 'l3'
BGPVPN_RES = "bgpvpns"
BGPVPN_TYPES = [BGPVPN_L3, BGPVPN_L2]
NETWORK_ASSOCIATION = 'network_association'
NETWORK_ASSOCIATIONS = 'network_associations'
ROUTER_ASSOCIATION = 'router_association'
ROUTER_ASSOCIATIONS = 'router_associations'

# The resource attribute map for the extension.
RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:uuid': None},
               'is_visible': True,
               'primary_key': True,
               'enforce_policy': True},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'validate': {
                          'type:string': db_const.PROJECT_ID_FIELD_SIZE},
                      'required_by_policy': True,
                      'is_visible': True,
                      'enforce_policy': True},
        'name': {'allow_post': True, 'allow_put': True,
                 'default': '',
                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                 'is_visible': True,
                 'enforce_policy': True},
        'type': {'allow_post': True, 'allow_put': False,
                 'default': BGPVPN_L3,
                 'validate': {'type:values': BGPVPN_TYPES},
                 'is_visible': True,
                 'enforce_policy': True},
        'route_targets': {'allow_post': True, 'allow_put': True,
                          'default': [],
                          'convert_to': converters.convert_to_list,
                          'validate': {'type:list_of_regex_or_none':
                                       RTRD_REGEX},
                          'is_visible': True,
                          'enforce_policy': True},
        'import_targets': {'allow_post': True, 'allow_put': True,
                           'default': [],
                           'convert_to': converters.convert_to_list,
                           'validate': {'type:list_of_regex_or_none':
                                        RTRD_REGEX},
                           'is_visible': True,
                           'enforce_policy': True},
        'export_targets': {'allow_post': True, 'allow_put': True,
                           'default': [],
                           'convert_to': converters.convert_to_list,
                           'validate': {'type:list_of_regex_or_none':
                                        RTRD_REGEX},
                           'is_visible': True,
                           'enforce_policy': True},
        'route_distinguishers': {'allow_post': True, 'allow_put': True,
                                 'default': [],
                                 'convert_to': converters.convert_to_list,
                                 'validate': {'type:list_of_regex_or_none':
                                              RTRD_REGEX},
                                 'is_visible': True,
                                 'enforce_policy': True},
        'networks': {'allow_post': False, 'allow_put': False,
                     'is_visible': True,
                     'enforce_policy': True},
        'routers': {'allow_post': False, 'allow_put': False,
                    'is_visible': True,
                    'enforce_policy': True}
    },
}

SUB_RESOURCE_ATTRIBUTE_MAP = {
    NETWORK_ASSOCIATIONS: {
        'parent': {'collection_name': COLLECTION_NAME,
                   'member_name': RESOURCE_NAME},
        'parameters': {
            'id': {'allow_post': False, 'allow_put': False,
                   'validate': {'type:uuid': None},
                   'is_visible': True,
                   'primary_key': True},
            'tenant_id': {'allow_post': True, 'allow_put': False,
                          'validate': {
                              'type:string': db_const.PROJECT_ID_FIELD_SIZE},
                          'required_by_policy': True,
                          'is_visible': True,
                          'enforce_policy': True},
            'network_id': {'allow_post': True, 'allow_put': False,
                           'validate': {'type:uuid': None},
                           'is_visible': True,
                           'enforce_policy': True}
        }
    },
    ROUTER_ASSOCIATIONS: {
        'parent': {'collection_name': COLLECTION_NAME,
                   'member_name': RESOURCE_NAME},
        'parameters': {
            'id': {'allow_post': False, 'allow_put': False,
                   'validate': {'type:uuid': None},
                   'is_visible': True,
                   'primary_key': True},
            'tenant_id': {'allow_post': True, 'allow_put': False,
                          'validate': {
                              'type:string': db_const.PROJECT_ID_FIELD_SIZE},
                          'required_by_policy': True,
                          'is_visible': True,
                          'enforce_policy': True},
            'router_id': {'allow_post': True, 'allow_put': False,
                          'validate': {'type:uuid': None},
                          'is_visible': True,
                          'enforce_policy': True}
        }
    }
}

ACTION_MAP = {
}

# The action status.
ACTION_STATUS = {
}

# The list of required extensions.
REQUIRED_EXTENSIONS = [l3.ALIAS]

# The list of optional extensions.
OPTIONAL_EXTENSIONS = [
]



# Copyright 2015-2016 Hewlett Packard Enterprise Development Company, LP
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import constants
from neutron_lib.api.definitions import l3
from neutron_lib.api.definitions import network

ALIAS = 'auto-allocated-topology'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Auto Allocated Topology Services'
API_PREFIX = ''
DESCRIPTION = 'Auto Allocated Topology Services.'
UPDATED_TIMESTAMP = '2016-01-01T00:00:00-00:00'
RESOURCE_NAME = 'auto_allocated_topology'
COLLECTION_NAME = 'auto_allocated_topologies'
RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:uuid': None},
               'is_visible': True},
        'tenant_id': {'allow_post': False, 'allow_put': False,
                      'validate': {'type:uuid': None},
                      'is_visible': True},
    },
    network.COLLECTION_NAME: {
        constants.IS_DEFAULT: {
            'allow_post': True,
            'allow_put': True,
            'default': False,
            'is_filter': True,
            'is_visible': True,
            'convert_to': converters.convert_to_boolean,
            'enforce_policy': True,
            'required_by_policy': True}},
}
SUB_RESOURCE_ATTRIBUTE_MAP = {}
ACTION_MAP = {}
REQUIRED_EXTENSIONS = [l3.ALIAS, 'subnet_allocation', 'external-net']
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# Copyright (c) 2018 Ericsson
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import port
from neutron_lib import constants


ALIAS = 'port-resource-request'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Port Resource Request'
DESCRIPTION = "Expose resource request to Port"
UPDATED_TIMESTAMP = "2018-05-08T10:00:00-00:00"
RESOURCE_NAME = port.RESOURCE_NAME
COLLECTION_NAME = port.COLLECTION_NAME
RESOURCE_REQUEST = 'resource_request'

RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {
        RESOURCE_REQUEST: {'allow_post': False,
                           'allow_put': False,
                           'default': constants.ATTR_NOT_SPECIFIED,
                           'is_visible': True}
    },
}

SUB_RESOURCE_ATTRIBUTE_MAP = None
ACTION_MAP = {}
ACTION_STATUS = {}
REQUIRED_EXTENSIONS = []
OPTIONAL_EXTENSIONS = []



# Copyright (c) 2018 Intel Corporation.
# Copyright (c) 2018 Isaku Yamahata <isaku.yamahata at intel com>
#                                   <isaku.yamahata at gmail com>
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


# NOTE(yamahata):smallest one is called first.
# we are using a big enough number for default that can
# be manipulated to increase or decrease the priorities.
# for all the callbacks, to reduce the priority division
# can be used for reducing that number for higher priority.
# each resources would want to define their own symbolic values for their use.

PRIORITY_DEFAULT = 55550000


# For l3 plugin and flavor
PRIORITY_ROUTER_EXTENDED_ATTRIBUTE = PRIORITY_DEFAULT - 100
# DEFAULT is reserved for third party which may not know priority yet
PRIORITY_ROUTER_DEFAULT = PRIORITY_DEFAULT
PRIORITY_ROUTER_CONTROLLER = PRIORITY_DEFAULT + 100
PRIORITY_ROUTER_DRIVER = PRIORITY_DEFAULT + 200
